#!/usr/bin/env perl
# process_request.cgi

#
# Process a request submitted from a MEME results form.
# Requests include:
#	Submit to MAST
#	View hidden field
#	Submit to BLOCKS
#	Submit to Meta-MEME
#	Print a man page
#
#
# Changes:
# print header ( Content-type:  text/html\n\n in the beginning of the script )
# this fixes the 'Error 500 malformed header from script' error
# 12/12/2002	JKP
# 03/24/2003	TLB; fixed problems with all types of submissions
#		
#
# USER SETTABLE LINKS
#
################################################################################
#
# You can change this if you wish to use a different BLOCKS server
$blocks_url = "http://blocks.fhcrc.org/blocks-bin/process_blocks.pl";
#
# You can change this if you wish to use a different Meta-MEME server
#$metameme_url = "http://metameme.sdsc.edu/beta";
$metameme_url = "http://metameme.sdsc.edu";
################################################################################

# requires
# use the CGI package
use CGI qw/:standard/;			# use the CGI package
use HTTP::Request::Common qw(POST);
use LWP::UserAgent;

# get the directories relative to working directory
#$root = "/rna/tbailey/web/meme";	# for kisac only
$root = "..";

#
# get the action (type of submit button pressed) and branch on it
#
$action = param('action');			# which submit button pressed
$nmotifs = param('nmotifs');			# number of motifs
#print header;

if ($action eq "MAST") {
  mast_search($nmotifs);
} elsif ($action =~ /^BLOCKS/) {
  submit_block(1..$nmotifs);
} elsif ($action =~ /^View (\w+) (\d+)/) {
  hidden_field($1, $2);
} elsif ($action =~ /^Submit BLOCK (\d+)/) {
  submit_block($1);
} elsif ($action =~ /^View motif summary/) {
  hidden_field('motif-summary');
} elsif ($action =~ /MetaMEME/){
  submit_metameme();
} elsif ($action =~ /MEME Man Page/){
  print_man("meme");
} elsif ($action =~ /MAST Man Page/){
  print_man("mast");
} else {
  print_header($action);			# print a response header
  print "action: $action\n";
  print "<H1>Feature not implemented yet.</H1>\n";
  exit(1);
}

# all done!
exit(0);

#
# submit a MetaMEME query
#
sub submit_metameme {
  print_header("Submit MetaMEME Query");
  # Get the contents of the MEME form and hash
  foreach $pname (param()) { $params{$pname} = param($pname); }
  # post the contents of the MEME form to MetaMEME
  $ua = LWP::UserAgent->new();
  my $req = POST "$metameme_url/cgi-bin/mhmm_process_request.cgi", [%params];
  my $request = $ua->request($req);
 # $content = $ua->request($req)->as_string;
  $content = $request->content;
  print $content;
} # submit_metameme

#
# Submit a MAST query
#
sub mast_search {
  my($nmotifs) = @_;

  print "Content-type: text/html\n\n";		# start form

  #
  # get the fields MAST needs
  #
  $url = param('url');
  $name = param('name');
  $alphabet = param('alphabet');
  for ($i=1; $i<=$nmotifs; $i++) {		# get the per-motif fields
    $block[$i] = param('BLOCKS'.$i);		# BLOCKS
    $pssm[$i] = param('pssm'.$i);		# pssm
    $pspm[$i] = param('pspm'.$i);		# pspm
  }

  #
  # check input
  #
  if ($nmotifs <= 0) {
    print "<PRE>There are no valid motifs in your MEME file.</PRE>\n";
    exit(1);
  }

  #
  # create a mast submission form by replacing the motif-file field
  # with the in-line motifs
  #

  # open the mast submission form
  open(MASTFORM, "< $root/mast.html") || 
    die("Cannot open MAST submission form mast.html: $!");

  # read the submission form and remove the motifs field and put in inline motifs
  while (<MASTFORM>) {
    if (/INPUT NAME=\"motifs\"/) {		# remove this line
      print "MEME results on <B>$name</B>\n";
      # put the inline motifs here
      print "<INPUT TYPE = HIDDEN NAME = inline_name VALUE = \"MEME motifs from sequences in $name\">\n";
      print "<INPUT TYPE = HIDDEN NAME = inline_motifs VALUE = \n\"";
      print "ALPHABET= $alphabet\n";
      for ($i=1; $i<=$nmotifs; $i++) { print(param('pssm'.$i)); }
      print "\">\n";
    } elsif (/^\s*ACTION =/) {			# use specified url for MAST
      print "ACTION = \"$url/website/cgi-bin/mast.cgi\"\n";
    } elsif (/\/HEAD/) {
      print "<BASE HREF=\"$url/website/\">";	# base url 
    } else {
      print;					# copy other lines
    }
  }
  close(MASTFORM);
} # mast_search

#
# view a hidden field
#
sub hidden_field {
  my($name, $number) = @_;

  print_header($action);
  if ($name eq "BLOCK") {
    $field = param('BLOCKS'.$number);		# get BLOCK
  } elsif ($name eq "FASTA") {
    $field = param('BLOCKS'.$number);
    $field = block2fasta($field);		# convert to FASTA
  } elsif ($name eq "RAW") {
    $field = param('BLOCKS'.$number);
    $field = block2raw($field);			# convert to raw format 
  } elsif ($name eq "PSSM") {
    $field = param('pssm'.$number);		# get PSSM
  } elsif ($name eq "PSPM") {
    $field = param('pspm'.$number);		# get PSPM
  } elsif ($name eq "motif-summary") {
    $field = param('motif-summary');		# get motif-summary
  } else { 
    print "Unknown hidden field type: $name\n";
    exit(1)
  }
  print "<PRE>$field</PRE>";
} # hidden_field

#
# Convert a BLOCK to RAW sequence format
#
sub block2raw {
  my($block) = @_;
  my($i, @lines, @words, $raw);

  $fasta = "";					# return value
  @lines = split(/\n/, $block); 		# split block into lines
  for ($i = 2; $i<$#lines; $i++) {
    @words = split(/\s+/, $lines[$i]);		# split line into words
    # get sequence line
    $raw .= "$words[3]\n";
  }
  $raw;
} # block2raw

#
# Convert a BLOCK to FASTA
#
sub block2fasta {
  my($block) = @_;
  my($i, @lines, @words, $fasta, $start);

  $fasta = "";					# return value
  @lines = split(/\n/, $block); 		# split block into lines
  for ($i = 2; $i<$#lines; $i++) {
    @words = split(/\s+/, $lines[$i]);		# split line into words
    # get id line and sequence line
    $start = substr($words[2], 0, length($words[2])-1);
    $fasta .= ">$words[0] ( start= $start )\n$words[3]\n";
  }
  $fasta;
} # block2fasta

#
# Submit a block to the blocks processor
#
sub submit_block {
  my(@numbers) = @_;
  my($blocks);

  print_header("Submit BLOCKS");
 
  # get the BLOCK(S)
  $blocks = "";
  foreach $number (@numbers) {
    $blocks .= param('BLOCKS'.$number);
  }

  $ua = LWP::UserAgent->new();
  my $req = POST $blocks_url, [ sequences => $blocks ];
  my $request = $ua->request($req);
  #$content = $ua->request($req)->as_string;
  $content = $request->content;
  # put in the absolute url's : this is FRAGILE!
  $content =~ s#HREF=\"#HREF=\"http://blocks.fhcrc.org#g;
  print $content;
} # submit_block

#
# Print a man page
#
sub print_man {
  my($command) = @_;
  print <<END; 
  Content-type: text/plain

END
  chdir("../../LOGS");
  $bin = "../bin";
  @tmp = `$bin/$command`;
  print @tmp[3..$#tmp];
}

#
# start a response form
#
sub print_header {
  my($action) = @_;
  print <<END; 
Content-type: text/html

<HTML>
<TITLE> MEME - $action</TITLE>
<BODY BACKGROUND=\"../bkg.jpg\">
END
} # print_header
